@@ -7,12 +7,47 @@ from .server_api import (
 
     get_server_api_connection,
 
+    raw_get,
+    raw_post,
+    raw_put,
+    raw_patch,
+    raw_delete,
+
     get,
     post,
     put,
     patch,
     delete,
 
+    download_file,
+    upload_file,
+
+    query_graphql,
+
+    get_addons_info,
+
+    get_dependencies_info,
+    update_dependency_info,
+
+    download_dependency_package,
+    upload_dependency_package,
+    delete_dependency_package,
+
+    get_full_production_settings,
+    get_production_settings,
+    get_full_project_settings,
+    get_project_settings,
+    get_addon_settings,
+
+    get_user,
+    get_users,
+
+    get_attributes_for_type,
+    get_default_fields_for_type,
+
+    get_project_anatomy_preset,
+    get_project_anatomy_presets,
+
     get_projects,
     get_project,
     create_project,
@@ -47,9 +82,6 @@ from .server_api import (
     get_representations_parents,
     get_representation_parents,
 
-    get_project_settings,
-    get_addon_settings,
-
     create_thumbnail,
     get_thumbnail,
     get_folder_thumbnail,
@@ -62,16 +94,50 @@ __all__ = (
     "ServerAPIBase",
 
     "ServerAPI",
+
     "get_server_api_connection",
+
+    "raw_get",
+    "raw_post",
+    "raw_put",
+    "raw_patch",
+    "raw_delete",
+
     "get",
     "post",
     "put",
     "patch",
     "delete",
 
+    "download_file",
+    "upload_file",
+
+    "query_graphql",
+
+    "get_addons_info",
+
+    "get_dependencies_info",
+    "update_dependency_info",
+
+    "download_dependency_package",
+    "upload_dependency_package",
+    "delete_dependency_package",
+
+    "get_full_production_settings",
+    "get_production_settings",
+    "get_full_project_settings",
     "get_project_settings",
     "get_addon_settings",
 
+    "get_user",
+    "get_users",
+
+    "get_attributes_for_type",
+    "get_default_fields_for_type",
+
+    "get_project_anatomy_preset",
+    "get_project_anatomy_presets",
+
     "get_projects",
     "get_project",
     "create_project",
@@ -89,6 +155,7 @@ __all__ = (
     "get_subset_by_name",
     "get_subsets",
     "get_subset_families",
+
     "get_version_by_id",
     "get_version_by_name",
     "version_is_latest",
