@@ -3,6 +3,7 @@ import re
 import json
 import logging
 import collections
+import platform
 import copy
 from http import HTTPStatus
 
@@ -36,6 +37,7 @@ from .exceptions import (
     UnauthorizedError,
     AuthenticationError,
     ServerNotReached,
+    ServerError,
 )
 from .utils import (
     logout_from_server,
@@ -151,10 +153,21 @@ class ServerAPIBase(object):
     Login cause that a session is used
 
     Args:
-        base_url(str): Example: http://localhost:5000
+        base_url (str): Example: http://localhost:5000
+        token (str): Access token (api key) to server.
+        site_id (str): Unique name of site. Should be the same when
+            connection is created from the same machine under same user.
+        client_version (str): Version of client application (used in
+            desktop client application).
     """
 
-    def __init__(self, base_url, token=None):
+    def __init__(
+        self,
+        base_url,
+        token=None,
+        site_id=None,
+        client_version=None
+    ):
         if not base_url:
             raise ValueError("Invalid server URL {}".format(str(base_url)))
 
@@ -164,6 +177,8 @@ class ServerAPIBase(object):
         self._graphl_url = "{}/graphql".format(base_url)
         self._log = None
         self._access_token = token
+        self._site_id = site_id
+        self._client_version = client_version
         self._access_token_is_service = None
         self._token_is_valid = None
         self._server_available = None
@@ -189,6 +204,20 @@ class ServerAPIBase(object):
     def access_token(self):
         return self._access_token
 
+    def get_site_id(self):
+        return self._site_id
+
+    def set_site_id(self, site_id):
+        if self._site_id == site_id:
+            return
+        self._site_id = site_id
+        # Recreate session on machine id change
+        if self._session is not None:
+            self.close_session()
+            self.create_session()
+
+    site_id = property(get_site_id, set_site_id)
+
     @property
     def is_server_available(self):
         if self._server_available is None:
@@ -213,7 +242,11 @@ class ServerAPIBase(object):
 
     def validate_token(self):
         try:
-            self.get_user_info()
+            # TODO add other possible validations
+            # - existence of 'user' key in info
+            # - validate that 'site_id' is in 'sites' in info
+            self.get_info()
+            self.get_user()
             self._token_is_valid = True
 
         except UnauthorizedError:
@@ -223,7 +256,7 @@ class ServerAPIBase(object):
     def set_token(self, token):
         self.reset_token()
         self._access_token = token
-        self.get_user_info()
+        self.get_user()
 
     def reset_token(self):
         self._access_token = None
@@ -258,6 +291,23 @@ class ServerAPIBase(object):
         self._session_functions_mapping = {}
         session.close()
 
+    def get_info(self):
+        """Get information about current used api key.
+
+        By default, the 'info' contains only 'uptime' and 'version'. With
+        logged user info also contains information about user and machines on
+        which was logged in.
+
+        Todos:
+            Use this method for validation of token instead of 'get_user'.
+
+        Returns:
+            Dict[str, Any]: Information from server.
+        """
+
+        response = self.get("info")
+        return response.data
+
     def _get_user_info(self):
         if self._access_token is None:
             return None
@@ -279,11 +329,23 @@ class ServerAPIBase(object):
         self._access_token_is_service = None
         return None
 
-    def get_user_info(self):
-        user_info = self._get_user_info()
-        if user_info is None:
+    def get_users(self):
+        # TODO how to find out if user have permission?
+        users = self.get("users")
+        return users.data
+
+    def get_user(self, username=None):
+        output = None
+        if username is None:
+            output = self._get_user_info()
+        else:
+            response = self.get("users/{}".format(username))
+            if response.status == 200:
+                output = response.data
+
+        if output is None:
             raise UnauthorizedError("User is not authorized.")
-        return user_info
+        return output
 
     @property
     def log(self):
@@ -294,7 +356,18 @@ class ServerAPIBase(object):
     def get_headers(self, content_type=None):
         if content_type is None:
             content_type = "application/json"
-        headers = {"Content-Type": content_type}
+
+        headers = {
+            "Content-Type": content_type,
+            "x-ayon-platform": platform.system().lower(),
+            "x-ayon-hostname": platform.node(),
+        }
+        if self._site_id is not None:
+            headers["x-ayon-site-id"] = self._site_id
+
+        if self._client_version is not None:
+            headers["x-ayon-version"] = self._client_version
+
         if self._access_token:
             if self._access_token_is_service:
                 headers["X-Api-Key"] = self._access_token
@@ -306,7 +379,7 @@ class ServerAPIBase(object):
     def login(self, username, password):
         if self.has_valid_token:
             try:
-                user_info = self.get_user_info()
+                user_info = self.get_user()
             except UnauthorizedError:
                 user_info = {}
 
@@ -471,6 +544,23 @@ class ServerAPIBase(object):
     def delete(self, entrypoint, **kwargs):
         return self.raw_delete(entrypoint, params=kwargs)
 
+    def download_file(self, endpoint, filepath, chunk_size=None):
+        if not chunk_size:
+            # 1 MB chunk by default
+            chunk_size = 1024 * 1024
+        dst_directory = os.path.dirname(filepath)
+        if not os.path.exists(dst_directory):
+            os.makedirs(dst_directory)
+
+        with open(filepath, "wb") as f_stream:
+            with self.raw_get(endpoint, stream=True) as response:
+                for chunk in response.iter_content(chunk_size=chunk_size):
+                    f_stream.write(chunk)
+
+    def upload_file(self, endpoint, filepath):
+        with open(filepath, "rb") as stream:
+            self.raw_post(endpoint, data=stream)
+
     def trigger_server_restart(self):
         result = self.post("system/restart")
         if result.status_code != 204:
@@ -673,6 +763,118 @@ class ServerAPIBase(object):
                 }
             )
 
+    def get_addons_info(self, details=True):
+        """Get information about addons available on server.
+
+        Args:
+            details (bool): Detailed data with information how to get client
+                code.
+        """
+
+        endpoint = "addons"
+        if details:
+            endpoint += "?details=1"
+        response = self.get(endpoint)
+        if response.status != 200:
+            raise ServerError(response.text)
+        return response.data
+
+    def get_dependencies_info(self):
+        result = self.get("dependencies")
+        return result.data
+
+    def update_dependency_info(
+        self,
+        name,
+        platform_name,
+        size,
+        checksum,
+        checksum_algorithm=None,
+        supported_addons=None,
+        python_modules=None,
+        sources=None
+    ):
+        """Update or create dependency package infor by it's identifiers.
+
+        The endpoint can be used to create or update dependency package.
+
+        Args:
+            name (str): Name of dependency package.
+            platform_name (Literal["windows", "linux", "darwin"]): Platform for
+                which is dependency package targeted.
+            size (int): Size of dependency package in bytes.
+            checksum (str): Checksum of archive file where dependecies are.
+            checksum_algorithm (str): Algorithm used to calculate checksum.
+                By default, is used 'md5' (defined by server).
+            supported_addons (Dict[str, str]): Name of addons for which was the
+                package created ('{"<addon name>": "<addon version>", ...}').
+            python_modules (Dict[str, str]): Python modules in dependencies
+                package ('{"<module name>": "<module version>", ...}').
+            sources (List[Dict[str, Any]]): Information about sources where
+                dependency package is available.
+        """
+
+        kwargs = {
+            key: value
+            for key, value in (
+                ("checksumAlgorithm", checksum_algorithm),
+                ("supportedAddons", supported_addons),
+                ("pythonModules", python_modules),
+                ("sources", sources),
+            )
+            if value
+        }
+
+        response = self.put(
+            "dependencies",
+            name=name,
+            platform=platform_name,
+            size=size,
+            checksum=checksum,
+            **kwargs
+        )
+        if response.status not in (200, 201):
+            raise ServerError("Failed to create/update dependency")
+        return response.data
+
+    def download_dependency_package(
+        self,
+        dst_directory,
+        package_name,
+        platform_name=None
+    ):
+        if platform_name is None:
+            platform_name = platform.system().lower()
+
+        package_filepath = os.path.join(dst_directory, package_name + ".zip")
+        self.download_file(
+            "dependencies/{}/{}".format(package_name, platform_name),
+            package_filepath
+        )
+        return package_filepath
+
+    def upload_dependency_package(
+        self, filepath, package_name, platform_name=None
+    ):
+        if platform_name is None:
+            platform_name = platform.system().lower()
+
+        self.upload_file(
+            "dependencies/{}/{}".format(package_name, platform_name),
+            filepath
+        )
+
+    def delete_dependency_package(self, package_name, platform_name=None):
+        if platform_name is None:
+            platform_name = platform.system().lower()
+
+        response = self.delete(
+            "dependencies/{}/{}".format(package_name, platform_name),
+        )
+        if response.status != 200:
+            raise ServerError("Failed to delete dependency file")
+        return response.data
+
     # Anatomy presets
     def get_project_anatomy_presets(self, add_default=True):
         result = self.get("anatomy/presets")
@@ -713,13 +915,32 @@ class ServerAPIBase(object):
             return full_settings
         return full_settings["settings"]
 
-    def get_addon_settings(self, addon_name, addon_version, project_name):
+    def get_addon_studio_settings(self, addon_name, addon_version):
+        result = self.get(
+            "addons/{}/{}/settings".format(addon_name, addon_version)
+        )
+        if result.status != 200:
+            raise ServerError(result.text)
+        return result.data
+
+    def get_addon_project_settings(
+        self, addon_name, addon_version, project_name
+    ):
         result = self.get(
             "addons/{}/{}/settings/{}".format(
-                addon_name, addon_version, project_name))
-        if result.status == 200:
-            return result.data
-        return None
+                addon_name, addon_version, project_name
+            )
+        )
+        if result.status != 200:
+            raise ServerError(result.text)
+        return result.data
+
+    def get_addon_settings(self, addon_name, addon_version, project_name=None):
+        if project_name is None:
+            return self.get_addon_studio_settings(addon_name, addon_version)
+        return self.get_addon_project_settings(
+            addon_name, addon_version, project_name
+        )
 
     # Entity getters
     def get_rest_project(self, project_name):
@@ -1864,6 +2085,7 @@ class ServerAPIBase(object):
             version_ids=[version_id],
             active=None,
             fields=fields,
+            own_attributes=own_attributes
         )
         for representation in representations:
             return representation
@@ -1919,7 +2141,7 @@ class ServerAPIBase(object):
         fields=None,
         own_attributes=False
     ):
-        filters = {}
+        filters = {"projectName": project_name}
         if task_ids is not None:
             task_ids = set(task_ids)
             if not task_ids:
@@ -1944,7 +2166,7 @@ class ServerAPIBase(object):
         if own_attributes:
             fields.add("ownAttrib")
 
-        query = workfiles_info_graphql_query()
+        query = workfiles_info_graphql_query(fields)
 
         for attr, filter_value in filters.items():
             query.set_variable_value(attr, filter_value)
